// 
//  Copyright (C) 1999-2003,2007,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef DS_ROWTABLE_H
#define DS_ROWTABLE_H

#ifndef DS_ROW_H
#include "dsRow.hh"
#endif

#include <vector>
using std::vector;

#ifndef LIBERR_H
#include <liberr.h>
#endif

extern "C" {
 
#ifndef DSLIB_H
#include "dslib.h"
#endif

#ifndef ASCDM_H
#include "ascdm.h"        /* AXAF Data Model lib */
#endif

#ifndef STKLIB_H
#include "dmstack.h"
#endif

#ifndef TRUE
#define TRUE (1)
#endif
#ifndef FALSE
#define FALSE (0)
#endif
}

#ifndef DS_DMDESCRIPTOR_H
#include "dsDMDescriptor.hh"
#endif


typedef struct dm_descr_array {
  dmDescriptor *descr_a[1024];
} dsDMDescrArray;

/* class structure that wraps datamodel calls around a stack of files,
   and a dsRow object */
class dsRowTable
{
public:

  /* constructors */
  dsRowTable(void);

  ~dsRowTable(void){};

  dsRowTable(const dsRowTable &copy){HelpClean(); HelpConstructor(copy);};

  /* operators */
  dsRowTable &operator = (const dsRowTable &right) {return *this;};

  /* modifiers */
  dsErrBool AddDatasets(Stack &input_files, dsErrList *err_list_p);

  dsErrBool SetRow(long new_row);
  
  dsErrBool AddElement(string new_element, dsErrList *err_list_p);

  dsErrBool AddRow(unsigned short new_location = dsBack);

  /* creation functions */
  dsErrBool CreateDataset(char *file_name_p,
			  char *block_name_p,
			  dsErrList *err_list_p);

  dsErrBool CreateCopy(char *file_name_p,
		       dmBlock *block_2_copy_p,
		       dsErrList *err_list_p);

  dsErrBool CreateElement(dsDMDescriptor *element_descr_p,
			  dsErrList *err_list_p);

  /* open all the columns that exist in a dataset */
  void OpenAllElements(void);

  dsErrBool CopyElement(dsRowTable *other,
			string element_name,
			dsErrList *err_list_p);

  /* Write row of data */
  void WriteRow(void);

  /* remove an element from the dsRowTable */
  void RemoveElem(string del_element);

  void RemoveAllElem(void);

  void RemoveRow(unsigned short new_location = dsFront);

  dsErrBool NextRow(void);
  
  dsErrBool PrevRow(void);

  dsErrBool SetType(string element, dmDataType new_type_e, long num_bits = 0);

  dsErrBool SetValue(string element,
		     dsAllDataTypes &new_data,
		     long index = 1,
		     double timepixr_val = 0.5 /*default*/,
		     double timedel_val = 0.0 /*default*/);
					  
  /* accessors */
  dmBlock *GetBlock(long N = 0) const {return block_ptrs[N];};

  long GetRow() const {return current_row_in_stack;};

  dsErrBool GetColName(long index, string &element);

  dsErrBool GetDSType(string element, dsDataTypes &holder_e);

  dsErrBool GetDMType(string element,dmDataType &holder_e);

  dsErrBool GetDSDMDescriptor(string element, dsDMDescriptor *&output_p);

  dsErrBool GetDSDimensions(string element, dsDimensions &output);

  dsErrBool GetUnit(string element, string &output);
  
  dsErrBool GetDescription(string element,string &output);
  
  dsErrBool GetElement(string element,dsAllDataTypes &holder_t,long index = 1);

  dsErrBool GetDescriptorType(string element,
			      dmDescriptorType &dm_descr_type_e);

  dsErrBool IsAnElement(string element) {dsToLower(element); return data.IsAnElement(element);};

  long GetNumCols() const {return num_columns;};

  long GetNumTableRows() const {return num_total_rows;};

  long GetNumRows() const {return data.GetNumRows();};

  long GetCurrentRow() const {return current_row_in_stack;};
  
  long GetLength(string element);

  string GetFileName(long index = 0) {return file_names[index];};

  void CloseDatasets(void);

  dsErrBool GetRange(string element, dsAllDataTypes &min, dsAllDataTypes &max);
protected:

  void HelpClean(void);
  void HelpConstructor(const dsRowTable &other);

  dmBlock *OpenDataset(char *file_name_p,
		       dsErrList *err_list_p);

  void CloseDataset(long index){dmTableClose(block_ptrs[index]);};

  short HelpNextRow(void);

  short HelpPrevRow(void);

  void WriteRowCell( string element, long row_index);

  void FillRow(void);

  void FillRowCell(string element, long row_index);

  dsErrBool OpenColumn(string &col_name, dsErrList *err_list_p);

  dsRow data;

  long current_file;
  long num_files;
  vector<string> file_names;
  vector<dmBlock *> block_ptrs;

  long num_columns;
  vector<dsDMDescriptor *> column_meta_data;
  vector<dsDMDescrArray> column_descriptors;

  bool isImage;

  long num_total_rows;
  long current_num_rows;
  long current_row_in_file;
  long current_row_in_stack;
  vector<long> num_rows_in_files;
  unsigned short location; /* either dsFront or dsBack - which row is the "one"
			      that current_row_in_file is pointing to? */

};

#endif

