/* -*- mode:C; mode: fold -*- */
/*
  This file is part of the Chandra Expmap program suite.
  Copyright (c) 1999-2012,2022 Massachusetts Institute of Technology

  This software was developed by the MIT Center for Space Research
  under contract SV1-61010 from the Smithsonian Institution.

  Permission to use, copy, modify, distribute, and sell this software
  and its documentation for any purpose is hereby granted without fee,
  provided that the above copyright notice appear in all copies and
  that both that copyright notice and this permission notice appear in
  the supporting documentation, and that the name of the Massachusetts
  Institute of Technology not be used in advertising or publicity
  pertaining to distribution of the software without specific, written
  prior permission.  The Massachusetts Institute of Technology makes
  no representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.

  THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE MASSACHUSETTS
  INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
  OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/
/* -*- mode:C; mode: fold -*- */

#ifndef _EXPMAP_EMAP_H_
#define _EXPMAP_EMAP_H_

#ifdef __cplusplus
extern "C" {
# if 0
}
# endif
#endif

#define EMAP_VERSION "0.8.3-0"
extern char *EMap_Version;

/*{{{ Parameter File interface */

#if USE_SAO_PFILE
/* This interface is a wrapper around the SAO interface such that the
 * MIT API can be used.
 */
typedef int *Param_File_Type;
typedef struct
{
   char *name;
   unsigned int type;
   void *value;
}
Param_Table_Type;
#define PF_INTEGER_TYPE		1
#define PF_BOOLEAN_TYPE		2
#define PF_REAL_TYPE		3      /* also DOUBLE type */
#define PF_DOUBLE_TYPE		4      /* also REAL type */
#define PF_STRING_TYPE		5
#define PF_COMMENT_TYPE		6
#define PF_FILE_TYPE		7
#define PF_INT_TYPE		PF_INTEGER_TYPE
#define PF_UINT_TYPE		8
#define PF_STRING0_TYPE		9

extern Param_File_Type *pf_parse_cmd_line (char *, char *, int, char **);
extern int pf_get_parameters (Param_File_Type *, Param_Table_Type *);
extern int pf_close_parameter_file (Param_File_Type *);
extern char *pf_get_input_filename (Param_File_Type *);
extern int pf_parameter_exists (Param_File_Type *, char *);
extern void emap_free_rmf_param_table(Param_Table_Type *table);

#else
/* Use MIT parameter file code */
# include <pfile.h>
#endif

extern char *emap_get_string_param (Param_File_Type *, char *);
extern char *emap_get_non_zero_string_param (Param_File_Type *, char *);

/*}}}*/

#include <ascdm.h>
extern int emap_write_ardlib_history (dmBlock *);
extern int emap_write_pixlib_history (dmBlock *);
extern int _emap_write_pixlib_history (int (*)(void *, char *), void *);

extern void emap_dm_error (char *file);
extern int emap_dm_write_key_string (dmBlock *b, char *key, char *value,
			      char *unit, char *comment, int *status);
extern int emap_dm_write_key_double (dmBlock *b, char *key, double value,
			     char *unit, char *comment, int *status);
extern int emap_dm_write_key_int (dmBlock *b, char *key, int value,
				  char *unit, char *comment, int *status);
extern int emap_dm_write_history (dmBlock *b, char *history, int *status);
extern int emap_dm_write_history_2 (dmBlock *b, char *kwd, char *val, int *status);
extern int emap_dm_write_section_start (dmBlock *b, char *name, int *status);
extern int emap_dm_write_comment (dmBlock *b, char *text, int *status);

extern int emap_dm_get_energy_grid (char *spec, float **lop, float **hip, float **midp, unsigned int *np);
int emap_dm_get_grid_d (char *spec, double **lop, double **hip, double **midp, unsigned int *np);

typedef struct _EMap_Hdrlib_Type EMap_Hdrlib_Type;
extern EMap_Hdrlib_Type *emap_get_hdrlib_headers (char *file, char *extname);
extern void emap_free_hdrlib_headers (EMap_Hdrlib_Type *h);
extern int emap_put_hdrlib_headers (dmBlock *b, EMap_Hdrlib_Type *h, int type);

/* FITS */
#include <sys/types.h>
#include <fitsio.h>

typedef struct
{
   int col;
   int data_type;
   long repeat;
#define EMAP_FITS_MAX_COLUMN_DIMS 16
   int num_dims;
   long dims[EMAP_FITS_MAX_COLUMN_DIMS];
}
Emap_Fits_Column_Info_Type;

//This function complete headers keywords for SAO std.  
extern int  emap_extra_putHdr(fitsfile *f, EMap_Hdrlib_Type *h, char * _tool, int basic_flag);

extern int emap_fits_write_key_string (fitsfile *f, char *key, char *value,
			      char *unit, char *comment, int *status);
extern int emap_fits_write_key_double (fitsfile *f, char *key, double value,
			     char *unit, char *comment, int *status);
extern int emap_fits_write_key_int (fitsfile *f, char *key, int value,
				  char *unit, char *comment, int *status);
extern int emap_fits_write_history (fitsfile *f, char *history, int *status);
extern int emap_fits_write_history_2 (fitsfile *f, char *kwd, char *val, int *status);
extern int emap_fits_write_section_start (fitsfile *f, char *name, int *status);
extern int emap_fits_write_comment (fitsfile *f, char *text, int *status);
extern void emap_fits_error (int status, char *fmt, ...);

extern int emap_fits_write_ardlib_history (fitsfile *);
extern int emap_fits_write_pixlib_history (fitsfile *);

extern fitsfile *emap_fits_open_binary_table (char *, char *, unsigned int *, unsigned int *);
extern int emap_fits_close_file (fitsfile *);
extern int emap_fits_read_key_string (fitsfile *, char *, char **);
extern int emap_fits_get_col_info (fitsfile *, char *, Emap_Fits_Column_Info_Type *);
extern int emap_fits_read_scol_in_row (fitsfile *, int, long, long, long, char **);
extern int emap_fits_read_icol_in_row (fitsfile *, int, long, long, long, int *);
extern int emap_fits_read_fcol_in_row (fitsfile *, int, long, long, long, float *);
extern int emap_fits_read_dcol_in_row (fitsfile *, int, long, long, long, double *);
extern int emap_fits_get_column_info (fitsfile *f, char *name, Emap_Fits_Column_Info_Type *ci);
extern int emap_fits_get_column_num (fitsfile *f, char *name, int *colnum);
extern int emap_fits_key_exists (fitsfile *f, char *name, int type, void *value);
extern int emap_fits_read_hduvers (fitsfile *f, char *keyword, int *x, int *y, int *z);

extern int emap_fits_read_key_double (fitsfile *f, char *file, char *key,
				      int is_required, double *valp);
extern int emap_fits_read_key_int (fitsfile *f, char *file, char *key,
				   int is_required, int *valp);

/*{{{ FitsIO interface */

typedef struct _Fitslib_Bin_Table_Type Fitslib_Bin_Table_Type;
typedef struct _Fitslib_Bin_Col_Info_Type Fitslib_Bin_Col_Info_Type;

extern Fitslib_Bin_Table_Type *
fitslib_open_bintable (char *,
		       unsigned int, char **col_names,
		       char *, int);

extern void fitslib_close_bintable (Fitslib_Bin_Table_Type *);
int fitslib_bintable_read_doubles (Fitslib_Bin_Table_Type *, double *);

extern int fitslib_get_num_bintable_rows (Fitslib_Bin_Table_Type *, unsigned int *);
extern int fitslib_bt_read_key_double (Fitslib_Bin_Table_Type *bt,
				       char *key, double *value,
				       int is_required);
extern int fitslib_bt_read_key_string (Fitslib_Bin_Table_Type *bt,
				       char *key, char **value,
				       int is_required);

#if 0
extern int fitslib_read_2d_fimage (char *, float **,
				   unsigned int *,
				   unsigned int *);

extern int fitslib_write_2d_fimage (char *, float *,
			     unsigned int,
			     unsigned int);

extern void fitslib_free (char *);
extern char *fitslib_malloc (unsigned int);
#endif

/*}}}*/

#if 0
/*{{{ Fits Image interface */

#define EM_CHAR_TYPE	1
#define EM_UCHAR_TYPE	2
#define EM_INT16_TYPE	3
#define EM_UINT16_TYPE	4
#define EM_INT32_TYPE	5
#define EM_UINT32_TYPE	6
#define EM_FLOAT32_TYPE	7
#define EM_FLOAT64_TYPE	8

typedef struct _EMap_FitsImage_Type EMap_FitsImage_Type;

extern EMap_FitsImage_Type *emap_fits_create_image (char *, unsigned int,
						    long *, int);

extern int emap_write_keyword_double (EMap_FitsImage_Type *, char *key, double *);
extern int emap_fits_close_image (EMap_FitsImage_Type *);
extern int emap_fits_write_image (EMap_FitsImage_Type *, float *);

extern int emfits_read_2d_image (char *, void **,
				 int,
				 unsigned int *,
				 unsigned int *,
				 EMap_WCS_Info_Type *);
/*}}}*/
#endif

/*{{{ Observation Info Interface */

typedef struct
{
   /* These come from a fits file. */
   char *mission;
   char *telescop;
   char *instrume;
   char *detnam;
   char *grating;
   char *filter;
   char *readmode;
   char *obsid;
   char *ds_ident;
   double sim_x, sim_y, sim_z;
#if 0
   char *date_obs;
   char *date_end;
   char *ardlib_date_string;
   /* Assumed to be of the form 1999-10-17T23:54:49;1999-10-17T23:54:49
    * where the ';' separates the time start from the time end -- used by ardlib.
    */
#endif
   /* These come from a parameter file */

   /* may include ardlib filters */
   char *mirror_subsystem;
   char *det_subsystem;
   char *subsystem_chipname;

   char *grating_arm;

   char *named_aimpoint;
   int cycle;			       /* value of CYCLE keyword in ACIS files */
}
EMap_Obs_Info_Type;

extern void emap_free_obs_info (EMap_Obs_Info_Type *);
extern EMap_Obs_Info_Type *emap_get_obs_info (char *, Param_File_Type *);
extern int emap_write_obs_info (dmBlock *, EMap_Obs_Info_Type *, int has_grating);
extern int emap_fits_write_obs_info (fitsfile *f, EMap_Obs_Info_Type *o, int has_grating);

/*}}}*/

/*{{{ Pixlib Interface */

typedef struct
{
   char *pixlib_chip_name;
   int chip_id;			       /* pixlib chip id */

   unsigned int num_x_pixels;
   unsigned int num_y_pixels;

   double focal_length;		       /* mm */

   /* Focal Plane (Tangent Plane Info) */

   double fp_pixel_size;	       /* degrees (yuk) */
   unsigned int fp_num_x_pixels;
   unsigned int fp_num_y_pixels;
   double fp_center_x;
   double fp_center_y;

   double fp_aimpoint_x;
   double fp_aimpoint_y;

   double grating_angle;	       /* radians */
   double grating_period;	       /* microns */
   double grating_node[3];	       /* location of Node in MNC coords */
}
EMap_Det_Info_Type;

extern EMap_Det_Info_Type *emap_init_pixlib (Param_File_Type *, EMap_Obs_Info_Type *);
extern void emap_close_pixlib (void);
extern void emap_free_detinfo (EMap_Det_Info_Type *);

extern int emap_set_bilinear_transform (double, double, double,
					double, double, double, double);
extern int emap_sky_to_chip (double, double, double *, double *);
extern int emap_chip_to_mnc (double *, double *);

extern void emap_mnc_to_theta_phi (double *mnc, double *theta, double *phi);

extern char *emap_get_pixlib_detnam (char *);
char *emap_get_canonical_detnam (char *);

extern int emap_chip_to_sky (double chip_x, double chip_y,
			     double ra_offset, double dec_offset, double roll,
			     double *sky_x, double *sky_y);

/*}}}*/

extern int emap_read_1d_f_data (char *, float **, unsigned int *);
extern int emap_read_2d_f_data (char *, float **, float **, unsigned int *);

typedef struct _EMap_Grating_Info_Type EMap_Grating_Info_Type;
extern int emap_init_psf_lib (Param_File_Type *);
extern void emap_close_psf_lib (void);

typedef struct _EMap_PSF_Type EMap_PSF_Type;
extern EMap_PSF_Type *emap_psf_open (EMap_Det_Info_Type *, EMap_Grating_Info_Type *);
extern int emap_psf_close (EMap_PSF_Type *);
extern int emap_psf_compute_fraction (EMap_PSF_Type *,
				      float, float, float *, float *, unsigned int,
				      float *, unsigned int, float *);

/*{{{ Aspect Histogram Interface */

typedef struct _EMap_AspHist_Type EMap_AspHist_Type;
typedef struct
{
   double ra, dec, roll;	       /* degrees, Sigh. */
   double dy, dz, theta;
   double delta_time;
   double tstart, tstop;
}
EMap_Aspect_Type;

extern int emap_rewind_asphist (EMap_AspHist_Type *);
extern void emap_close_asphist (EMap_AspHist_Type *);
extern int emap_get_aspect_offset (EMap_AspHist_Type *, EMap_Aspect_Type *);
extern EMap_AspHist_Type *emap_open_asphist (char *);
extern int emap_asphist_nominal_pointing (EMap_AspHist_Type *, double *, double *, double *);
extern int emap_asphist_get_num_points (EMap_AspHist_Type *, unsigned int *);
extern int emap_asphist_get_ave_offset (EMap_AspHist_Type *h, EMap_Aspect_Type *);

/*}}}*/

/*{{{ Some utility functions */

extern char *emap_realloc (char *, unsigned int);
extern char *emap_malloc (unsigned int);
extern void emap_free (char *);
extern int emap_strcasecmp (char *, char *);
extern char *emap_make_string (char *);
extern char *emap_make_nstring (char *, unsigned int);
extern int emap_strncasecmp (char *, char *, unsigned int);
extern char *emap_trim_string (char *s);
extern char *emap_skip_white (char *s);
extern char *emap_get_fits_date (void);
extern char *emap_getcwd (void);
extern char *emap_make_cmdline_string (char **, unsigned int);

extern int emap_check_clobber (char *file, int clob, int verb);

extern int emap_parse_dbl_range_spec2 (char *range_spec,
				       double *xminp, double *xmaxp,
				       double *dxp, unsigned int *numxp,
				       double *yminp, double *ymaxp,
				       double *dyp, unsigned int *numyp);
extern int emap_parse_uint_range_spec2 (char *range_spec,
					unsigned int *xminp, unsigned int *xmaxp,
					double *dxp, unsigned int *numxp,
					unsigned int *yminp, unsigned int *ymaxp,
					double *dyp, unsigned int *numyp);
typedef struct
{
   double inv_dx, inv_dy;	       /* Inverse length of sizes */
   double x,y;			       /* lower left corner */
#define EMAP_MAX_BILINEAR_DIMS	10
   unsigned int num_dims;
   double f0[EMAP_MAX_BILINEAR_DIMS];
   double f1[EMAP_MAX_BILINEAR_DIMS];
   double f2[EMAP_MAX_BILINEAR_DIMS];
   double f3[EMAP_MAX_BILINEAR_DIMS];
}
EMap_Bilinear_Interp_Type;

extern int emap_bilinear_init (EMap_Bilinear_Interp_Type *,
			      double, double,   /* lower left corner coords */
			      double, double,   /* length of sizes */
			      unsigned int,   /* num-dims */
			      double *, /* Values at LL corner */
			      double *, /* Values at LR corner */
			      double *, /* Values at UL corner */
			      double * /* Values at UR corner */);
extern int emap_bilinear_interp (EMap_Bilinear_Interp_Type *,
				double, double,   /* coords of point */
				double *);   /* result(s) */

extern int emap_interpolate_fvector (float *, float *, unsigned int,
				     float *, float *, unsigned int);
extern unsigned int emap_binary_search_f (float, float *, unsigned int);
extern float emap_interpolate_f (float, float *, float *, unsigned int);
extern unsigned int emap_binary_search_d (double, double *, unsigned int);
extern double emap_interpolate_d (double, double *, double *, unsigned int);

extern void emap_reverse_dvector (double *, unsigned int);

/*}}}*/

extern int emap_set_program_name (char *name);
extern char *emap_get_program_name (void);
extern void emap_free_program_name (void);

typedef struct _EMap_Det_Exposure_Type EMap_Det_Exposure_Type;
extern void emap_close_det_exposure (EMap_Det_Exposure_Type *);
extern EMap_Det_Exposure_Type *emap_open_det_exposure (EMap_Obs_Info_Type *, Param_File_Type *);
extern int emap_get_det_exposure (EMap_Det_Exposure_Type *, double, double, double *);

extern int emap_set_verbose (int);

#ifdef PI
#undef PI
#endif
#define PI 3.14159265358979323846264338327950288

#ifdef __cplusplus
# if 0
{
# endif
}
#endif

#endif				       /* _EXPMAP_EMAP_H_ */
