/*                                                                
**  Copyright (C) 1996,1998,2000-2002,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*H****************************************************************************

* FILE NAME: matrixlib.h
 
* DEVELOPEMENT: tools
 
* DESCRIPTION:

  Header file for the matrixlib library.  This header file must be included in 
  any programs attempting to use the pixlib library. 

* NOTES:
 
  This header file is used by the library. It can be used for  applications,too.
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
         1.1             05/30/96
 
*H****************************************************************************/

#ifndef _MATRIXLIB_H_
#define _MATRIXLIB_H_

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define free_and_null(x) {  free((char*)(x)); (x) = NULL;}

short mth_unit_matrix(long nrows, long ncols, double **);
short mth_zero_vector(long nrows, double *);
short mth_zero_matrix(long nrows, long ncols,  double **);

short mth_matrix_multiple_vector(long, long, double **, double *, double *);
short mth_matrix_multiple_matrix(long, long, double **, long, long, double **, double **);
short mth_rotate_matrix( long nrows, long ncols, double rot_angle, double **m);
short mth_rotate_plane(double *a, double roll_angle, double *b);
short mth_compose_matrix(long, long, double **, double *, double **);
short mth_transpose_matrix(long, long, double **,  double **);
short mth_scale_matrix(long, long, double **, double, double **);
short mth_scale_vector(long, double *, double, double *);
short mth_copy_matrix(long nrows, long ncols, double **, double **);

short mth_vector_minus_vector(long, double *, double *, double *);
short mth_vector_plus_vector(long, double *, double *, double *);
short mth_matrix_minus_matrix(long, long, double **, double **, double **);
short mth_matrix_plus_matrix(long, long, double **, double **, double **);


short mth_form_unit_vector(long rows, double *a, double *b, double *c);
short mth_vector_x_vector(  long nrows, double *a, double *b, double *c);
short mth_vector_dot_vector(long nrows, double *v1, double *v2, double *scalar);
double mth_get_vector_length(  long nrows, double *a);

char **mth_alloc_cmatrix(long nrow, long ncol);
double **mth_alloc_dmatrix(long nrow, long ncol);
float **mth_alloc_fmatrix(long nrow, long ncol);
void mth_free_cmatrix(long nrow, char **m);
void mth_free_fmatrix(long nrow, float **m);
void mth_free_dmatrix(long nrow, double **m);

#endif /* _MATRIXLIB_H_ */
