// 
//  Copyright (C) 1999-2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)rmf_general.hh
 
* DEVELOPEMENT: Chandra X-ray Science Center fef library
                hhe@cfa.harvard.edu (Initial version)
 
* DESCRIPTION:  Header file

 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             08/10/98
 
*H****************************************************************************/

#ifndef  _RMF_GENERAL_HH_
#define  _RMF_GENERAL_HH_

#include "rmf_string.hh"

#include <iostream>
#include <fstream>
#include <iomanip>
#include <iterator>


extern "C" {
 #include "liberr.h"
 #include "dslib.h"
}

#ifndef  TRUE
#define  TRUE            1==1               /* TRUE  boolean flag */
#endif

#ifndef  FALSE
#define  FALSE           0==1               /* FALSE boolean flag */
#endif

// macros
#ifndef MAX
#define MAX(A,B) ((A) > (B) ? (A) : (B))
#endif
#ifndef MIN
#define MIN(A,B) ((A) < (B) ? (A) : (B))
#endif


//-------------------------------------------------
// generic class for debugging printout
//-------------------------------------------------
class DebugPrint
{


public:
  DebugPrint():d_fname(NULL){;};
  ~DebugPrint(){};

  void Pause();
  void PrintArray(      char *name, vector<long> array);
  void PrintArray(const char *name, vector<long> array){PrintArray( (char*)name,array);};


  void PrintArray(      char *name, vector<short> array);
  void PrintArray(const char *name, vector<short> array){PrintArray( (char*)name,array);};

  void PrintArray(      char *name, vector<float> array);
  void PrintArray(const char *name, vector<float> array){PrintArray( (char*)name,array);};

  void PrintArray(      char *name, long len, long *array);
  void PrintArray(const char *name, long len, long *array){PrintArray( (char*)name, len, array);};

  void PrintArray(      char *name, long len, short *array);
  void PrintArray(const char *name, long len, short *array){PrintArray( (char*)name, len, array);};

  void PrintArray(      char *name, long len, float *array);
  void PrintArray(const char *name, long len, float *array){ PrintArray( (char*)name, len, array);};

  void SetFName(      char *name){ d_fname = name;};
  void SetFName(const char *name){ d_fname = (char*)name;};

  void FprintArray(      char *name, vector<float> array);
  void FprintArray(const char *name, vector<float> array){FprintArray((char *)name, array);};

  void FprintArray(      char *name, long len, float *array);
  void FprintArray(const char *name, long len, float *array){FprintArray((char *)name, len, array);};

  void FprintArray(      char *name, vector<long> array);
  void FprintArray(const char *name, vector<long> array){FprintArray((char *)name, array);};

  void FprintArray(      char *name, long len, long *array);
  void FprintArray(const char *name, long len, long *array){FprintArray((char *)name, len, array);};

  void FprintArray(      char *name, long len, short *array);
  void FprintArray(const char *name, long len, short *array){FprintArray((char *)name, len, array);};

  void FprintArray(      char *name, vector<short> array);
  void FprintArray(const char *name, vector<short> array){FprintArray((char *)name, array);};

private:
  char *d_fname;
};
//
// Error class
//
class RMFError
{

public:
  RMFError()
  {      
    d_dsErrCde = dsNOERR;
    d_dsErrGrp = dsDBGRPERR;
    d_dsErrLst = NULL;
    d_dsErrIns = NULL;
    d_ErrlibInit = FALSE;
  };
  RMFError(dsErrList *_errLst)
  {      
    d_dsErrCde = dsNOERR;
    d_dsErrGrp = dsDBGRPERR;
    d_dsErrLst = _errLst;
    d_dsErrIns = NULL;
    d_ErrlibInit = FALSE;
  };
  ~RMFError(){ if(d_ErrlibInit == TRUE) dsErrCloseLib(); };
 
  void InitErrLib( dsErrGroup,  char *func_name );

  dsErrList *GetErrList(){  return d_dsErrLst;};
  dsErrGroup GetErrGroup(){ return d_dsErrGrp;};
  void SetErrList(dsErrList *_list){d_dsErrLst = _list;};
  void SetErrGroup(dsErrGroup _grp){d_dsErrGrp = _grp;};

protected:

  dsErrList *d_dsErrLst;
  dsErrGroup d_dsErrGrp;

  dsErrCode d_dsErrCde;
  dsErrInstance *d_dsErrIns;

private:
  bool d_ErrlibInit;
};
//-------------------------------------------------
// generic class of rmf general macros or definition
//-------------------------------------------------
class RMFGeneral 
: public RMFError, public DebugPrint
{

public:
  RMFGeneral(): RMFError(), DebugPrint(), d_log(NULL), d_verbose(0){};
  RMFGeneral(dsErrList *_errLst, FILE *_log, short _verb=0): 
    RMFError(_errLst), DebugPrint(), d_log(_log), d_verbose(_verb) {};
  ~RMFGeneral(){};

  enum ERROR {Success, Warning, Fail,
	      FileNotFound, NoDimension, AxisNameNotFound,
	      ParameterNotFound, RangeErr, SizeErr, DimensionErr, 
	      InconsistentDimension, FailToSetFeFObj, FailToSetUserObj,
	      ExceedDimensionLimit, FailToFormEffGrids,
	      ExceedSystemMemory, ReadFileFail, UserGridErr,
	      NullPointer, FITSWriteFail, FITSCloseFail, 
	      MATRIXFail, EBFail,
	      NotSupportedBinMethod,
	      APIFail,
	      OpenFail, BadFile, EmptyData, ReadFail, WriteFail,
	      GetKeywordValueFail,
	      SetKeywordValueFail,
	      NotSupportedErr
  };
  enum LOCAL_MACROS{ USER_FLAG, FEF_FLAG, 
		     MATRIX=0, EBOUNDS=1, NUM_EXTNSON=2,
		     RMF_MAX_DIMN = 2,
		     MAX_KEYWD_LEN = 8,
		     MAX_STRING_LEN = DS_SZ_LINE,
		     MAX_BUFFER_SZ = 1024,
		     MAX_MEMORY_SZ = MAX_BUFFER_SZ*MAX_BUFFER_SZ
  };

  void DeAllocate2DMemory(char **m);
  char **Allocate2DMemory(long nrows, long ncols);

  void SetLogPointer(FILE *_log) {d_log = _log;};
  void SetVerbose( short _verb){   d_verbose = _verb;};
  bool isinteger(float fnum)
  {
    long num = (long)fnum;
    if( fnum - num  < 1.e-9 )
      return TRUE;
    else
      return FALSE;
  };

  FILE     *d_log;// log the process(error or warning
                   // also log the debug process externally controlled
                   // be verbose parameter 
  short d_verbose; // debug level(0-5): = 0 no debug
                   //                     1-5, debug level from low to highest

};


#endif //_RMF_GENERAL_HH_
