set -e

# To keep things flexible, as we might want to install CIAO e.g. in $PREFIX/ciao
export CIAO_PREFIX=${PREFIX}

export CFLAGS="`echo $CFLAGS | sed s/-O2//g`"
export CXXFLAGS="`echo $CXXFLAGS | sed s/-O2//g`"
export CPPFLAGS="`echo $CPPFLAGS | sed s/-O2//g`"
export LDFLAGS="`echo $LDFLAGS | sed s/-Wl,-O2//g`"

if [ "`uname -s`" == "Darwin" ] ; then
  if [ "$(uname -m)" != "arm64" ] ; then
    CONDA_BUILD_SYSROOT="${MACOSX_SDK:-/opt/MacOSX10.14.sdk}"
  else
    CONDA_BUILD_SYSROOT="${MACOSX_SDK:-/opt/MacOSX11.0.sdk}"
  fi

  # Hacks for commands called directly from Makefiles
  ln -sf $CC $BUILD_PREFIX/bin/gcc

  CONFIG_EXTRA="--x-includes=/opt/X11/include --x-libraries=/opt/X11/lib"

  #Issue with competing X11 libs (from tk)
  rm -rvf $PREFIX/include/X11
  rm -rvf $CONDA_PREFIX/include/X11

  CONFIG_EXTRA="${CONFIG_EXTRA} CXXFLAGS=-std=c++14"

  #Added for fix-in.sh
  OSXONLY=""
  LINUXONLY="#"
else
  # Hacks for commands called directly from Makefiles
  ln -sf $GCC $BUILD_PREFIX/bin/gcc
  CONFIG_EXTRA="CXXFLAGS=-std=gnu++14"
  OSXONLY="#"
  LINUXONLY=""
fi

#Set the --with-pyversion to the major.minor python version
export CIAO_PY_VER=${CONDA_PY:0:1}.${CONDA_PY:1:2}
if [ "x$CONDA_PY" != "x" ] ; then
  CONFIG_EXTRA="$CONFIG_EXTRA --with-pyversion=$CIAO_PY_VER"
else
  echo "Error: CONDA_PY not set. Should be set if Conda provides Python."
  exit 1
fi

# Build
./configure --with-ver="CIAO 4.16" --disable-maintainer-mode --with-fits --with-ascii --with-pic --with-opt=-O3 grplib_python_includes="-I${CIAO_PREFIX}/include -L${CIAO_PREFIX}/lib" --prefix=${CIAO_PREFIX} --with-pkgconfig="${PREFIX}" --with-ots=${CIAO_PREFIX} ${CONFIG_EXTRA}
make install
#Make -k should NOT be used. This is only temporary to get around build issues.
#make -k install || true

# Make sure ciaover works.
echo "conda list | grep 'ciao \|ciao-contrib \|caldb\|^#\|sherpa \|ds9 \|bkg_evt' ; echo \"\" ; echo \"System information:\" ;  uname -a" > ${CIAO_PREFIX}/bin/ciaover
chmod +x ${CIAO_PREFIX}/bin/ciaover

touch $CIAO_PREFIX/bin/ciao.sh
touch $CIAO_PREFIX/bin/ciao_setup.sh

#Configure/add init/wrapper scripts
cat ${RECIPE_DIR}/fix-in.sh | sed "s%@OSXONLY@%${OSXONLY}%g" | sed "s%@LINUXONLY@%${LINUXONLY}%g" | sed "s%@CIAO_PY_VER@%${CIAO_PY_VER}%g" > $CIAO_PREFIX/bin/fix-in.sh
cp -Rv ${RECIPE_DIR}/init $CIAO_PREFIX/bin/init
cat $CIAO_PREFIX/bin/init/python | sed "s%binexe/python%binexe/python${CIAO_PY_VER}%g" > $CIAO_PREFIX/bin/init/python${CIAO_PY_VER}
chmod +x $CIAO_PREFIX/bin/init/python${CIAO_PY_VER} $CIAO_PREFIX/bin/fix-in.sh
rm $CIAO_PREFIX/bin/init/python

# Copy the [de]activate scripts to $PREFIX/etc/conda/[de]activate.d.
# This will allow them to be run on environment activation.
for CHANGE in "activate" "deactivate"
do
    mkdir -p "${PREFIX}/etc/conda/${CHANGE}.d"
    cp "${RECIPE_DIR}/${CHANGE}.sh" "${PREFIX}/etc/conda/${CHANGE}.d/${PKG_NAME}_${CHANGE}.sh"
    cp "${RECIPE_DIR}/${CHANGE}.csh" "${PREFIX}/etc/conda/${CHANGE}.d/${PKG_NAME}_${CHANGE}.csh"
done


#Create a list of all of the parameter files
# Used by the post-link.sh and pre-unlink.sh scripts
# to remove and add write permissions respectively
cd ${PREFIX}/param
find * -maxdepth 0 -type f -name "*.par" -exec echo {} \; > ${PREFIX}/param/.param.ciao
