# 
#  Copyright (C) 2007,2009-2010,2018  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


lappend auto_path . $env(OBSVIS_PKG_PATH)


package require Tktable
package require OBSVIS 

variable globalTestCaseList
variable globalTestFileName
variable globalGenerateStateFiles

proc RunUnitTest { inIndex inModuleName inCaseList} {

    set status {}
    set stat "PASSED"
    puts -nonewline "*********** Unit Test #$inIndex '"
    puts "$inModuleName' ***********"
#   set tclStatus [ catch { ${inModuleName}::SelfTest    }  status  ]
#   set tclStatus {}

    set globalTestCaseList $inCaseList

    if { [namespace which -command ${inModuleName}::SelfTest] == "" } {
	puts "'${inModuleName}::SelfTest' not found"
	set status {0 0}
	
    } else {
	set status [${inModuleName}::SelfTest ]
    }

    if { [lindex $status 0] != [lindex $status 1] || [lindex $status 1]  == 0 } {
	set stat "FAILED"
    } 

    puts -nonewline "*********** Unit Test #$inIndex '"
    puts "$inModuleName' $stat ******\n"

    return $status
}

proc RunRegressionTest { inIndex inTest inCaseList} {
    global env
    global globalTestFileName
    global globalTestCaseList
    global globalGenerateStateFiles

    
    set globalGenerateStateFiles $env(OBSVIS_GENERATE_STATE_FLAG)

    set status {}
    set stat "PASSED"
    puts -nonewline "*********** Regression Test #$inIndex '"
    puts "$inTest' ***********"


    set okFlag 1
    if { [file exists $env(OBSVIS_TEST_PATH)/OBSVIS_REGRESSIONTEST_${inTest}.tk ] != 1 } {
	puts "'$env(OBSVIS_TEST_PATH)/${inTest}' not found"
	set status {0 0}
	set okFlag 0
    } elseif { [file readable $env(OBSVIS_TEST_PATH)/OBSVIS_REGRESSIONTEST_${inTest}.tk ] != 1 } {
	puts "'$env(OBSVIS_TEST_PATH)/${inTest}' not readable"
	set status {0 0}
	set okFlag 0
    }

    if { $okFlag == 1 } {

	set globalTestCaseList $inCaseList
	set globalTestFileName OBSVIS_REGRESSIONTEST_${inTest}
	set status [source $env(OBSVIS_TEST_PATH)/OBSVIS_REGRESSIONTEST_${inTest}.tk ]
    }

    if { [lindex $status 0] != [lindex $status 1] || [lindex $status 1]  == 0 } {
	set stat "FAILED"
    } 
    
    puts -nonewline "*********** Regression Test #$inIndex '"
    puts "$inTest' $stat ******\n"

    return $status
}

proc RunTestSuit { inTestSuit inTestList} {
    global env
    global globalTestCaseList

    set OBSVISPreferences::defaults(gui,interactivemode) 0

    set allTestList {}
    if { $inTestSuit == "unit" } {
	
	foreach test [namespace children :: OBSVIS*] {
	    regsub {::} $test {} module
	    lappend allTestList $module
	} 

	set TestProcName "RunUnitTest"
	set TestType "UNIT"
    } else {

	foreach test [glob -nocomplain $env(OBSVIS_TEST_PATH)/OBSVIS_REGRESSIONTEST_*.tk] {
	    set test [file root [file tail $test]]
	    regsub OBSVIS_REGRESSIONTEST_ $test {} testCase
	    lappend allTestList $testCase
	}

	set TestProcName "RunRegressionTest"
	set TestType "REGRESSION"
    }

    # sort the list to ensure indexing doesn't change
    set allTestList [lsort $allTestList]

    set testList {}
    if { [llength $inTestList] == 0 } {
	set testList $allTestList
    } else {

	foreach test $inTestList {
	    
	    regsub -all {^.*:} $test {} testCaseString 
	    
	    if { $testCaseString == $test } {
		set testCaseString ""
	    } else {
		regsub {:.*$} $test {} test
		
		
	    }

	    set newTestCaseList {}

	    foreach testcase [split $testCaseString ,] {

		if { [regexp {^[0-9]+[-][0-9]+$} $testcase] == 1 } {
		    regsub {[-][0-9]+$} $testcase {} startIndx
		    regsub {^[0-9]+[-]} $testcase {} endIndx

		    if { $startIndx > $endIndx } {
			set tmpVal $startIndx
			set startIndx $endIndx
			set endIndx $tmpVal
		    }

		    for {set indx $startIndx} { $indx <= $endIndx } {incr indx} {
			lappend  newTestCaseList $indx

		    }

		} else  {

		    lappend  newTestCaseList $testcase
		}
		
	    }



	    if { [regexp {^[0-9]+$} $test] == 1 } {

		if { $test > 0 && $test <= [llength $allTestList] } {
		    set test [lindex  $allTestList [expr $test -1] ]
		}

		if { [lsearch -glob $testList $test] == -1 } { 
		    lappend testList $test
		}

		if { [ info exists testCaseList($test)] != 1 } {
		    set testCaseList($test) {}
		}
		
		foreach testCase  $newTestCaseList {
		    if { [lsearch -exact $testCaseList($test)  $testCase] == -1 } {
			lappend testCaseList($test) $testCase
		    }
		}		

	    } else {
		set tmpList $allTestList
		
		while { 1 } {
		    
		    set indx [lsearch -glob $tmpList $test]

		    if { $indx != -1 } {

			set newtest [lindex $tmpList $indx]
			lappend testList $newtest
			
			if { [ info exists testCaseList($newtest)] != 1 } {
			    set testCaseList($newtest) {}
			}
			
			foreach testCase  $newTestCaseList {
			    if { [lsearch -exact $testCaseList($newtest)  $testCase] == -1 } {
				lappend testCaseList($newtest) $testCase
			    }
			}
			set tmpList [lreplace $tmpList $indx $indx]
		    } else {
			if { [lsearch -exact $testList $test] == -1 } {
			    lappend testList $test
			}
			break
		    }
		    
		    
		}
	    }
	}

    }

    set totalTestCount 0
    set successTestCount 0
    set totalCaseCount 0
    set successCaseCount 0
    set testSummaryList {}

    foreach test $testList {
	
	set module [file root [file tail $test]] 
	
	set testIndex [expr [lsearch -exact $allTestList $module ] +1 ]
	
	
	
	if { $testIndex == 0 } {
	    set sumEntry "#? Invalid Test '$module' "
	    set status {0 0}
	} else {
	    
	    if { [ info exists testCaseList($test)] != 1 } {
		set testCaseList($test) {}
	    }
	    set sumEntry "#$testIndex $module "

	    set status [$TestProcName $testIndex $module $testCaseList($test)]
	} 
	
	set totalCount [lindex $status 0]
	set succCount [lindex $status 1]
	incr totalCaseCount $totalCount
	incr successCaseCount $succCount
	
	if {  $totalCount == $succCount && $totalCount > 0 } {
	    incr successTestCount
	    append sumEntry "PASSED"
	} else {
	    append sumEntry "FAILED($succCount/$totalCount)"
	}
	
	lappend testSummaryList $sumEntry
	incr totalTestCount
	
    } 
    
    
    puts "$TestType TESTING SUMMARY: "
    foreach entry $testSummaryList {
	puts $entry
    }
    
    puts "TOTAL $successTestCount out of $totalTestCount tests passed\n\n"
}



proc IncludeTestCase { inTestCase inTestCaseList } {

    if { [llength $inTestCaseList ] == 0 } {
	return 1
    }

    if { [lsearch -exact $inTestCaseList $inTestCase ] != -1 } {

	return 1
    }

    return 0
}



proc CheckTestCase { inSourceDir inDestDir inFileName inSaveState } {
    global env

    set status 0
    OBSVISGui::SaveState $env(OBSVISHOME)/${inFileName}.state
    catch { 
	[exec diff $inDestDir/${inFileName}.state  $inSourceDir/${inFileName}.state |  sed s/>/EXPECTED:/ | sed s/</ACTUAL:/ >& $inDestDir/${inFileName}.diff ] 
    }
    
    file stat $inDestDir/${inFileName}.diff fileInfo

    if { $inSaveState != 1 } { 
	file delete $inDestDir/${inFileName}.state
    }

    if { [ info exists fileInfo] == 1 } { 
	if { $fileInfo(size) == 0 } { 
	    file delete $inDestDir/${inFileName}.diff
	    set status 1
	} else {
	    
	}
    }
    

    return $status
}

proc TestInternetConnection {} {

   set internetFound 1
   OBSVISPreferences::New 
   
   if {[ catch { set token [http::geturl "$OBSVISPreferences::defaults(application,calibration,url)" -timeout $OBSVISPreferences::defaults(application,calibration,timeout)] } ] > 0  } {
      set internetFound 0
   }

   return $internetFound
}


if { [ info exists env(OBSVISHOME) ] != 1} {
    set env(OBSVISHOME) $env(HOME)
}

set noExitFlag 0
set testOptions ""
set internetConnection 1
if { [info exists env(OBSVIS_TESTS_OPTIONS) ] == 1 } {
    set testOptions $env(OBSVIS_TESTS_OPTIONS)
} 

set testList {}
if { [info exists env(OBSVIS_TESTS) ] == 1 } {
    set testList $env(OBSVIS_TESTS)
} 



foreach option $testOptions {

    switch $option {
        "testinternet" {
            set internetConnection [TestInternetConnection]
        }
	"listregression" {
	    puts "***** Available tests:"
	    set indx 1
	    foreach test [glob -nocomplain $env(OBSVIS_TEST_PATH)/OBSVIS_REGRESSIONTEST_*.tk] {
		regsub OBSVIS_REGRESSIONTEST_ $test {} testCase
		puts "$indx [file root [file tail $testCase]]"
		incr indx
	    }
	    
	}
	"listunit" {
	   puts "***** Available tests:"
	    set indx 1
	    foreach test [namespace children :: OBSVIS*] {
		regsub {::} $test {} module
		puts "$indx $module"
		incr indx
	    } 
	}
	"runregressionnoexit" {
	    set noExitFlag 1
	    RunTestSuit "regression" $testList
	}
	"runregression" {
	    RunTestSuit "regression" $testList
	}
	"rununit" {
            if { $internetConnection == 1 } {
	       RunTestSuit "unit" $testList
            } else {
               puts "SKIPPING: Obsvis smoke test needs an internet connection to run."
            }
	}
	
    }
    
}

if { $noExitFlag != 1 } {
    QuitDS9
}



